/*=========================================================================*/

#include <gpib.h>
#include <utility.h>
#include <formatio.h>
#include "cy_if11.h"

/*= Cytec IF11 IEEE488 Control Module ======================================*/
/* LabWindows Instrument Driver                                            */
/* Original Release: Jan. 1994                                             */
/* By: R. Hopf                                                             */
/* Originally written in C                                                 */
/* Modification History: None                                              */
/*=========================================================================*/

/*= INSTRUMENT TABLE ======================================================*/
/* address array: contains the GPIB addresses of opened instruments.       */
/* bd array: contains the device descriptors returned by OpenDev.          */
/* instr_cnt: contains the number of instruments open of this model type.  */
/*=========================================================================*/
static int address[IF11_MAX_INSTR + 1];
static int bd[IF11_MAX_INSTR + 1];
static int instr_cnt;

/*= STATIC VARIABLES ======================================================*/
/* cmd is a buffer for GPIB I/O strings.                                   */
/* if11_err: the error variable for the instrument module                */
/* ibcnt: contains the number of bytes transferred by GPIB reads and       */
/*        writes.  See the GPIB library I/O Class for more information     */
/*=========================================================================*/
static char cmd[50];
static int if11_err;

/*= UTILITY ROUTINES ======================================================*/
int if11_open_instr (int, int *);
int if11_close_instr (int);
int if11_invalid_integer_range (int, int, int, int);
int if11_device_closed (int);
int if11_read_data (int, char *, int);
int if11_write_data (int, char *, int);
int if11_set_timeout (int, int, int *);
void if11_setup_arrays (void);

/*=========================================================================*/
/* Function: Initialize                                                    */
/* Purpose:  This function opens the instrument, queries the instrument    */
/*           for its ID, and initializes the instrument to a known state.  */
/*=========================================================================*/
int if11_init (addr, rest, instrID)
int   addr;
int   rest;
int * instrID;
{
	int  ID;

	if (if11_invalid_integer_range (addr, 0, 30, -1) != 0)
		return if11_err;
	if (if11_invalid_integer_range (rest, 0, 1, -3) != 0)
		return if11_err;

	if (if11_open_instr (addr, &ID) != 0)
		return if11_err;

	if (rest) {
		if (if11_write_data (ID, "C", 1) != 0) {
			if11_close_instr (ID);
			return if11_err;
		}
		Delay(0.01);
	}
	if11_setup_arrays ();
	*instrID = ID;

	return if11_err;
}

/*=========================================================================*/
/* - Operations: Latch, Unlatch, Multiplex, Clear and Status --- */

int if11_operate (instrID, Operation, Module, Relay, Status)
int instrID, Operation, Module, Relay;
int *Status;
{
	char s[20];

	*Status = -1;
	if (Operation == 'C') {
		Fmt(s,"C");
		if (if11_write_data(instrID, s, StringLength(s)) != 0)
			return if11_err;
		Delay(0.01);
	}
	else {
		Fmt(s,"%c %d %d", Operation, Module, Relay);
		if (if11_write_data(instrID, s, StringLength(s)) != 0)
			return if11_err;
	}
	if (if11_read_data(instrID, s, 2) != 0)
		return if11_err;
	*Status = s[0] & 0xf;
	return if11_err;
}

/*=========================================================================*/
/* Function: Write To Instrument                                           */
/* Purpose:  This function writes a command string to the instrument.      */
/*=========================================================================*/
int if11_write (instrID, cmd_string)
int   instrID;
char *cmd_string;
{
	if (if11_invalid_integer_range (instrID, 1, IF11_MAX_INSTR, -1) != 0)
		return if11_err;
	if (if11_device_closed(instrID) != 0)
		return if11_err;

	Fmt (cmd, "%s<%s", cmd_string);
	if (if11_write_data (instrID, cmd, NumFmtdBytes()) != 0)
		return if11_err;

	return if11_err;
}

/*=========================================================================*/
/* Function: Read Instrument Buffer                                        */
/* Purpose:  This function reads the output buffer of the instrument.      */
/*=========================================================================*/
int if11_read (instrID, numbytes, in_buff, bytes_read)
int   instrID;
int   numbytes;
char *in_buff;
int  *bytes_read;
{
	if (if11_invalid_integer_range (instrID, 1, IF11_MAX_INSTR, -1) != 0)
		return if11_err;
	if (if11_device_closed(instrID) != 0)
		return if11_err;

	*bytes_read = 0;
	if (if11_read_data (instrID, in_buff, numbytes) != 0)
		return if11_err;

	*bytes_read = ibcnt;

	return if11_err;
}

/*=========================================================================*/
/* Function: Close                                                         */
/* Purpose:  This function closes the instrument.                          */
/*=========================================================================*/
int if11_close (instrID)
int instrID;
{
	if (if11_invalid_integer_range (instrID, 1, IF11_MAX_INSTR, -1) != 0)
		return if11_err;
	if (if11_device_closed (instrID))
		return if11_err;

	if11_close_instr (instrID);

	return if11_err;
}

/*= UTILITY ROUTINES ======================================================*/

/*=========================================================================*/
/* Function: Open Instrument                                               */
/* Purpose:  This function locates and initializes an entry in the         */
/*           Instrument Table and the GPIB device table for the            */
/*           instrument.  The size of the Instrument Table can be changed  */
/*           in the include file by altering the constant                  */
/*           IF11_MAX_INSTR.  The return value of this function is equal */
/*           to the global error variable.                                 */
/*=========================================================================*/
int if11_open_instr (addr, ID)
int  addr;
int *ID;
{
	int i, instrID;

	instrID = 0;
	if11_err = 0;

/* Check to see if the instrument is already in the Instrument Table. */

	for (i = 1; i <= IF11_MAX_INSTR; i++)
		if (address[i] == addr) {
			instrID = i;
			i = IF11_MAX_INSTR;
		 }

/* If it is not in the instrument table, open an entry for the instrument. */

	if (instrID <= 0)
		for (i = 1; i <= IF11_MAX_INSTR; i++)
			if (address[i] == 0) {
				instrID = i;
				i = IF11_MAX_INSTR;
			 }

/* If an entry could not be opened in the Instrument Table, return an error.*/

	if (instrID <= 0) {
		if11_err = 220;
		return if11_err;
	}

/* If the device has not been opened in the GPIB device table (bd[ID] = 0),*/
/*  then open it.                                                          */

	if (bd[instrID] <= 0) {
		if (instr_cnt <= 0)
			CloseInstrDevs("if11");
		bd[instrID] = OpenDev ("", "if11");
		if (bd[instrID] <= 0) {
			if11_err = 220;
			return if11_err;
		}
		instr_cnt += 1;
		address[instrID] = addr;
	 }

/*  Change the primary address of the device    */

	if (ibpad (bd[instrID], addr) < 0) {
		if11_err = 233;
		return if11_err;
	}

	*ID = instrID;
	return if11_err;
}

/*=========================================================================*/
/* Function: Close Instrument                                              */
/* Purpose:  This function closes the instrument by removing it from the   */
/*           GPIB device table and setting the address and bd[instrID] to  */
/*           zero in the Instrument Table.  The return value is equal to   */
/*           the global error variable.                                    */
/*=========================================================================*/
int if11_close_instr (instrID)
int instrID;
{
	if (bd[instrID] != 0) {
		CloseDev (bd[instrID]);
		bd[instrID] = 0;
		address[instrID] = 0;
		instr_cnt -= 1;
	}
	else
		if11_err = 221;

	return if11_err;
}

/*=========================================================================*/
/* Function: Invalid Integer Range                                         */
/* Purpose:  This function checks an integer to see if it lies between a   */
/*           minimum and maximum value.  If the value is out of range, set */
/*           the global error variable to the value err_code.  If the      */
/*           value is OK, error = 0.  The return value is equal to the     */
/*           global error value.                                           */
/*=========================================================================*/
int if11_invalid_integer_range (val, min, max, err_code)
int val;
int min;
int max;
int err_code;
{
	if (val < min || val > max)
		if11_err = err_code;
	else
		if11_err = 0;

	return if11_err;
}

/*=========================================================================*/
/* Function: Device Closed                                                 */
/* Purpose:  This function checks to see if the module has been            */
/*           initialized.  If the device has not been opened, set the      */
/*           global error variable to 232, 0 otherwise. The return value   */
/*           is equal to the global error value.                           */
/*=========================================================================*/
int if11_device_closed (instrID)
int instrID;
{
	if (bd[instrID] <= 0)
		if11_err = 232;
	else
		if11_err = 0;

	return if11_err;
}

/*=========================================================================*/
/* Function: Read Data                                                     */
/* Purpose:  This function reads a buffer of data from the instrument. The */
/*           return value is equal to the global error variable.           */
/*=========================================================================*/
int if11_read_data (instrID, buf, cnt)
int   instrID;
char *buf;
int   cnt;
{
	if (ibrd(bd[instrID], buf, (long)cnt) <= 0)
		if11_err = 231;
	else
		if11_err = 0;

	return if11_err;
}

/*=========================================================================*/
/* Function: Write Data                                                    */
/* Purpose:  This function writes a buffer of data to the instrument. The  */
/*           return value is equal to the global error variable.           */
/*=========================================================================*/
int if11_write_data (instrID, buf, cnt)
int   instrID;
char *buf;
int   cnt;
{
	if (ibwrt(bd[instrID], buf, (long)cnt) <= 0)
		if11_err = 230;
	else
		if11_err = 0;

	return if11_err;
}

/*=========================================================================*/
/* Function: Set Timeout                                                   */
/* Purpose:  This function changes or disables the timeout of the device.  */
/*           Refer to the LabWindows Standard Libraries Reference Manual   */
/*           for timeout codes. The return value is equal to the global    */
/*           error variable.                                               */
/*=========================================================================*/
int if11_set_timeout (instrID, tmo_code, old_timeout)
int  instrID;
int  tmo_code;
int *old_timeout;
{
	*old_timeout = ibtmo (bd[instrID], tmo_code);
	if (ibsta <= 0)
		if11_err = 239;
	else
		if11_err = 0;

	return if11_err;
}

/*=========================================================================*/
/* Function: Setup Arrays                                                  */
/* Purpose:  This function is called by the init routine to initialize     */
/*           static arrays.                                                */
/*           This routine should be modified for each instrument to        */
/*           include instrument-dependent commmand arrays.                 */
/*=========================================================================*/
void if11_setup_arrays ()
{
}		   
/*=== THE END =============================================================*/
